/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/

/*
 * Concepts:
 * !Creating a server!
 */

//CContextClientSession.cpp

#include "ContextCommon.h"
#include "ContextClientSession.h"
#include <e32math.h>
#include "mutexrequest.h"
#include "server_startup.h"
#include "break.h"
#include "cbbsession.h"
#include "bbtypes.h"

static const TUint KDefaultMessageSlots = 2;
static const TUid KServerUid3 = {0x10200C6B}; // matches UID in server/group/contextserver.mbm file

_LIT(KContextServerFilename, "ContextServer");

EXPORT_C RContextClientSession::RContextClientSession() : RSessionBase(), iTimeBuffer(NULL, 0, 0)
{
	
}

EXPORT_C TInt RContextClientSession::ConnectToContextServer()
{
	TInt result;
	
	CC_TRAP(result, StartServerL(KContextServerName, KServerUid3, KContextServerFilename));
	if (result == KErrNone)
	{
		result = CreateSession(KContextServerName,
			Version(),
			KDefaultMessageSlots);	
	}
	return result;
}

EXPORT_C TVersion RContextClientSession::Version() const
{
	return(TVersion(KContextServMajorVersionNumber,
		KContextServMinorVersionNumber,
		KContextServBuildVersionNumber));
}

//----------------------------------------------------------------------------------------

EXPORT_C void RContextClientSession::Cancel() const
{
	SendReceive(ECancel, NULL);
}

EXPORT_C void RContextClientSession::MsgTerminateContextServer(TRequestStatus& aStatus)
{
	SendReceive(ETerminateContextServer, NULL, aStatus);
}

EXPORT_C void RContextClientSession::MsgConnectToPresenceServer(const TDesC & username, 
								const TDesC & password, 
								const TDesC & server, TUint32 accessPointID, TRequestStatus& aStatus)
{
	TAny* messageParameters[KMaxMessageArguments];
	messageParameters[0] = (void*)&username;
	messageParameters[1] = (void*)&password;
	messageParameters[2] = (void*)&server;

	apPackage()=accessPointID;
	messageParameters[3] = &apPackage;
	SendReceive(EConnectToPresenceServer, &messageParameters[0], aStatus);
}

EXPORT_C void RContextClientSession::MsgRequestPresenceInfo(TDes & contact, TDes & presenceInfo, TTime & send_timestamp, TRequestStatus& aStatus)
{
	iTimeBuffer.Set(reinterpret_cast<TUint8*>(&send_timestamp), sizeof(send_timestamp), sizeof(send_timestamp));
	
	TAny* messageParameters[KMaxMessageArguments];
	messageParameters[0] = & contact;
	messageParameters[1] = & presenceInfo;
	messageParameters[2] = static_cast<TAny*>(&iTimeBuffer);
	
	SendReceive(ERequestPresenceInfo, &messageParameters[0], aStatus);	
}

EXPORT_C void RContextClientSession::MsgRequestMessageNotification(TDes & contact, 
								   TDes& subject, 
								   TDes & message, 
								   TRequestStatus& aStatus)
{
	TAny* messageParameters[KMaxMessageArguments];
	messageParameters[0] = & contact;
	messageParameters[1] = & subject;
	messageParameters[2] = & message;
	
	SendReceive(ERequestMessageNotification, &messageParameters[0], aStatus);
}

EXPORT_C void RContextClientSession::MsgRequestPresenceNotification(TDes & contact, TDes & presenceInfo, TTime & send_timestamp, TRequestStatus& aStatus)
{
	iTimeBuffer.Set(reinterpret_cast<TUint8*>(&send_timestamp), sizeof(send_timestamp), sizeof(send_timestamp));
	
	
	TAny* messageParameters[KMaxMessageArguments];
	messageParameters[0] = & contact;
	messageParameters[1] = & presenceInfo;
	messageParameters[2] = static_cast<TAny*>(&iTimeBuffer);

	SendReceive(ERequestPresenceNotification, &messageParameters[0], aStatus);
}

EXPORT_C void RContextClientSession::MsgUpdateUserPresence(TDesC & info, TRequestStatus& aStatus)
{
	TAny* messageParameters[KMaxMessageArguments];
	messageParameters[0] = & info;
	SendReceive(EUpdateUserPresence, &messageParameters[0], aStatus);
}

EXPORT_C void RContextClientSession::MsgSendMessage(const TDesC& contact, const TDesC& subject, 
	const TDesC& message, TRequestStatus& aStatus)
{
	TRequestStatus* s=&aStatus;
	User::RequestComplete(s, KErrNotSupported);
}

EXPORT_C void RContextClientSession::MsgSuspendConnection(TRequestStatus& aStatus)
{
	SendReceive(ESuspendConnection, NULL, aStatus);
}

EXPORT_C void RContextClientSession::MsgResumeConnection(TRequestStatus& aStatus)
{
	SendReceive(EResumeConnection, NULL, aStatus);
}


EXPORT_C void SendLookupNotificationL(class CBBSubSession* aBBSession,
			const TDesC& aJabberNick, const class MBBData* PresenceData)
{
	//FIXME
	return;

	TTime exp; exp.HomeTime(); exp+=TTimeIntervalDays(14);
	aBBSession->PutRequestL(KOutgoingLookupTuple, aJabberNick, PresenceData,
		exp, KNoComponent);
}
