#include "socketslistener.h"

#include "break.h"
#include "app_context.h"
#include "in_sock.h"
#include "symbian_auto_ptr.h"

class CSocketListenerImpl: public CSocketListener, public MContextBase {
	CSocketListenerImpl(MSocketAcceptorFactory& aAcceptorFactory);
	void ConstructL(TUint aListenPort);
	~CSocketListenerImpl();

	void CheckedRunL();
	void DoCancel();
	TInt CheckedRunError();

	void RestartL();

	TInt	iErrorCount;

	enum TState { EIdle, EAccepting };
	TState iState;

	RSocketServ iServ;
	RSocket	iSocket;
	MSocketAcceptor* iCurrentAcceptor;
	MSocketAcceptorFactory &iFactory;
	TInetAddr                   iAddress;

	friend CSocketListener;
	friend auto_ptr<CSocketListenerImpl>;
};

_LIT(KSocketListener, "CSocketListener");

CSocketListener::CSocketListener() : CCheckedActive(CActive::EPriorityLow, KSocketListener) { }

CSocketListenerImpl::CSocketListenerImpl(MSocketAcceptorFactory& aAcceptorFactory) : 
	iFactory(aAcceptorFactory) { }

EXPORT_C CSocketListener* CSocketListener::NewL(MSocketAcceptorFactory& aAcceptorFactory,
	TUint aListenPort)
{
	auto_ptr<CSocketListenerImpl> ret(new (ELeave) CSocketListenerImpl(aAcceptorFactory));
	ret->ConstructL(aListenPort);
	return ret.release();
}

void CSocketListenerImpl::ConstructL(TUint aListenPort)
{
	User::LeaveIfError(iServ.Connect());
	iAddress.SetAddress(INET_ADDR(127,0,0,1));
	iAddress.SetPort(aListenPort);
	RestartL();
}

CSocketListenerImpl::~CSocketListenerImpl()
{
	Cancel();
	iSocket.Close();
	iServ.Close();
	delete iCurrentAcceptor;
}

void CSocketListenerImpl::CheckedRunL()
{
	switch(iState) {
	case EAccepting:
		iCurrentAcceptor->AcceptFinished(iStatus.Int());
		iCurrentAcceptor=0;
		iCurrentAcceptor=iFactory.CreateAcceptorL();
		iSocket.Accept(iCurrentAcceptor->GetSocket(), iStatus);
		SetActive();
		break;
	}
}

void CSocketListenerImpl::DoCancel()
{
	switch(iState) {
	case EAccepting:
		iSocket.CancelAccept();
		delete iCurrentAcceptor; iCurrentAcceptor=0;
	}
}

TInt CSocketListenerImpl::CheckedRunError()
{
	TInt err;
	CC_TRAP(err, RestartL());
	return err;
}

void CSocketListenerImpl::RestartL()
{
	Cancel();
	iErrorCount++;
	iSocket.Close();
	User::LeaveIfError(iSocket.Open(iServ, KAfInet, KSockStream, KProtocolInetTcp));
	User::LeaveIfError(iSocket.Bind(iAddress));
	delete iCurrentAcceptor; iCurrentAcceptor=0;
	iCurrentAcceptor=iFactory.CreateAcceptorL();
	iSocket.Accept(iCurrentAcceptor->GetSocket(), iStatus);
	SetActive();
}